// Traveling Salesman Problem via SetBuilder Algorithm
// Written by David J. Stein, Esq., Fall 2004. See "License.html" for terms and conditions of use.
//
// This module contains the front-end code for the Traveling Salesman Problem.

#region Using declarations

using System;
using System.Drawing;
using System.Threading;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

#endregion

namespace Set_Builder_Algorithm_Namespace {

	public class TravelingSalesmanProblem : System.Windows.Forms.Form	{

		#region Variables, initialization code, and Main

		private System.Windows.Forms.PictureBox picMap;
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.Button btnClearNetwork;
		private System.Windows.Forms.ListView lstLevels;
		private System.Windows.Forms.Button btnSolveStep;
		private System.Windows.Forms.ColumnHeader columnHeader1;
		private System.Windows.Forms.ColumnHeader columnHeader2;
		private System.Windows.Forms.ColumnHeader columnHeader3;
		private System.Windows.Forms.Button btnInitialize;
		private System.Windows.Forms.Button btnSolveFull;
		private System.Windows.Forms.Button btnScaleCities;
		private System.Timers.Timer timerRefresh;
		private System.Windows.Forms.ColumnHeader columnHeader4;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.NumericUpDown nudSize;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.NumericUpDown nudThreshold;
		private System.Windows.Forms.TextBox txtStatus;
		private System.Timers.Timer timerStatusText;

		private Thread thread = null;
		private SetBuilderAlgorithm network;
		private City movingCity = null;
		private int iMouseX = 0, iMouseY = 0;

		public TravelingSalesmanProblem() {
			InitializeComponent();
			network = new SetBuilderAlgorithm();
		}

		protected override void Dispose(bool disposing)		{
			if (disposing) {
				if (components != null)  {
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(TravelingSalesmanProblem));
			this.picMap = new System.Windows.Forms.PictureBox();
			this.btnClearNetwork = new System.Windows.Forms.Button();
			this.btnScaleCities = new System.Windows.Forms.Button();
			this.btnSolveStep = new System.Windows.Forms.Button();
			this.lstLevels = new System.Windows.Forms.ListView();
			this.columnHeader1 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader2 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader3 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader4 = new System.Windows.Forms.ColumnHeader();
			this.btnInitialize = new System.Windows.Forms.Button();
			this.btnSolveFull = new System.Windows.Forms.Button();
			this.timerRefresh = new System.Timers.Timer();
			this.label1 = new System.Windows.Forms.Label();
			this.nudSize = new System.Windows.Forms.NumericUpDown();
			this.nudThreshold = new System.Windows.Forms.NumericUpDown();
			this.label2 = new System.Windows.Forms.Label();
			this.txtStatus = new System.Windows.Forms.TextBox();
			this.timerStatusText = new System.Timers.Timer();
			((System.ComponentModel.ISupportInitialize)(this.timerRefresh)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.nudSize)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.nudThreshold)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.timerStatusText)).BeginInit();
			this.SuspendLayout();
			// 
			// picMap
			// 
			this.picMap.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.picMap.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.picMap.Location = new System.Drawing.Point(8, 8);
			this.picMap.Name = "picMap";
			this.picMap.Size = new System.Drawing.Size(680, 464);
			this.picMap.TabIndex = 16;
			this.picMap.TabStop = false;
			this.picMap.Paint += new System.Windows.Forms.PaintEventHandler(this.picMap_Paint);
			this.picMap.MouseUp += new System.Windows.Forms.MouseEventHandler(this.picMap_MouseUp);
			this.picMap.MouseMove += new System.Windows.Forms.MouseEventHandler(this.picMap_MouseMove);
			this.picMap.MouseLeave += new System.EventHandler(this.picMap_MouseLeave);
			this.picMap.MouseDown += new System.Windows.Forms.MouseEventHandler(this.picMap_MouseDown);
			// 
			// btnClearNetwork
			// 
			this.btnClearNetwork.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.btnClearNetwork.Location = new System.Drawing.Point(296, 504);
			this.btnClearNetwork.Name = "btnClearNetwork";
			this.btnClearNetwork.TabIndex = 20;
			this.btnClearNetwork.Text = "Clear";
			this.btnClearNetwork.Click += new System.EventHandler(this.btnClearNetwork_Click);
			// 
			// btnScaleCities
			// 
			this.btnScaleCities.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.btnScaleCities.Location = new System.Drawing.Point(376, 504);
			this.btnScaleCities.Name = "btnScaleCities";
			this.btnScaleCities.TabIndex = 19;
			this.btnScaleCities.Text = "Scale";
			this.btnScaleCities.Click += new System.EventHandler(this.btnScaleCities_Click);
			// 
			// btnSolveStep
			// 
			this.btnSolveStep.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.btnSolveStep.Location = new System.Drawing.Point(536, 504);
			this.btnSolveStep.Name = "btnSolveStep";
			this.btnSolveStep.TabIndex = 21;
			this.btnSolveStep.Text = "Solve (Step)";
			this.btnSolveStep.Click += new System.EventHandler(this.btnSolveStep_Click);
			// 
			// lstLevels
			// 
			this.lstLevels.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.lstLevels.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																												this.columnHeader1,
																												this.columnHeader2,
																												this.columnHeader3,
																												this.columnHeader4});
			this.lstLevels.Location = new System.Drawing.Point(696, 8);
			this.lstLevels.Name = "lstLevels";
			this.lstLevels.Size = new System.Drawing.Size(256, 520);
			this.lstLevels.TabIndex = 22;
			this.lstLevels.View = System.Windows.Forms.View.Details;
			// 
			// columnHeader1
			// 
			this.columnHeader1.Text = "Level";
			// 
			// columnHeader2
			// 
			this.columnHeader2.Text = "Total";
			// 
			// columnHeader3
			// 
			this.columnHeader3.Text = "Best";
			// 
			// columnHeader4
			// 
			this.columnHeader4.Text = "Average";
			// 
			// btnInitialize
			// 
			this.btnInitialize.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.btnInitialize.Location = new System.Drawing.Point(456, 504);
			this.btnInitialize.Name = "btnInitialize";
			this.btnInitialize.TabIndex = 23;
			this.btnInitialize.Text = "Initialize";
			this.btnInitialize.Click += new System.EventHandler(this.btnInitialize_Click);
			// 
			// btnSolveFull
			// 
			this.btnSolveFull.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.btnSolveFull.Location = new System.Drawing.Point(616, 504);
			this.btnSolveFull.Name = "btnSolveFull";
			this.btnSolveFull.TabIndex = 24;
			this.btnSolveFull.Text = "Solve (Full)";
			this.btnSolveFull.Click += new System.EventHandler(this.btnSolveFull_Click);
			// 
			// timerRefresh
			// 
			this.timerRefresh.Enabled = true;
			this.timerRefresh.Interval = 500;
			this.timerRefresh.SynchronizingObject = this;
			this.timerRefresh.Elapsed += new System.Timers.ElapsedEventHandler(this.timerRefresh_Elapsed);
			// 
			// label1
			// 
			this.label1.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.label1.Location = new System.Drawing.Point(8, 504);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(64, 23);
			this.label1.TabIndex = 25;
			this.label1.Text = "Population";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// nudSize
			// 
			this.nudSize.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.nudSize.Location = new System.Drawing.Point(72, 504);
			this.nudSize.Maximum = new System.Decimal(new int[] {
																					 1000000,
																					 0,
																					 0,
																					 0});
			this.nudSize.Minimum = new System.Decimal(new int[] {
																					 1,
																					 0,
																					 0,
																					 0});
			this.nudSize.Name = "nudSize";
			this.nudSize.Size = new System.Drawing.Size(56, 20);
			this.nudSize.TabIndex = 26;
			this.nudSize.Value = new System.Decimal(new int[] {
																				  500,
																				  0,
																				  0,
																				  0});
			this.nudSize.ValueChanged += new System.EventHandler(this.nudSize_ValueChanged);
			// 
			// nudThreshold
			// 
			this.nudThreshold.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.nudThreshold.DecimalPlaces = 3;
			this.nudThreshold.Location = new System.Drawing.Point(208, 504);
			this.nudThreshold.Maximum = new System.Decimal(new int[] {
																							1,
																							0,
																							0,
																							0});
			this.nudThreshold.Name = "nudThreshold";
			this.nudThreshold.Size = new System.Drawing.Size(72, 20);
			this.nudThreshold.TabIndex = 28;
			this.nudThreshold.Value = new System.Decimal(new int[] {
																						 5,
																						 0,
																						 0,
																						 65536});
			this.nudThreshold.ValueChanged += new System.EventHandler(this.nudThreshold_ValueChanged);
			// 
			// label2
			// 
			this.label2.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.label2.Location = new System.Drawing.Point(144, 504);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(64, 23);
			this.label2.TabIndex = 27;
			this.label2.Text = "Threshold";
			this.label2.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// txtStatus
			// 
			this.txtStatus.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.txtStatus.Location = new System.Drawing.Point(8, 480);
			this.txtStatus.Name = "txtStatus";
			this.txtStatus.ReadOnly = true;
			this.txtStatus.Size = new System.Drawing.Size(680, 20);
			this.txtStatus.TabIndex = 29;
			this.txtStatus.Text = "Status: Ready";
			// 
			// timerStatusText
			// 
			this.timerStatusText.Enabled = true;
			this.timerStatusText.SynchronizingObject = this;
			this.timerStatusText.Elapsed += new System.Timers.ElapsedEventHandler(this.timerStatusText_Elapsed);
			// 
			// TravelingSalesmanProblem
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(960, 534);
			this.Controls.Add(this.txtStatus);
			this.Controls.Add(this.nudThreshold);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.nudSize);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.btnSolveFull);
			this.Controls.Add(this.btnInitialize);
			this.Controls.Add(this.lstLevels);
			this.Controls.Add(this.btnSolveStep);
			this.Controls.Add(this.btnClearNetwork);
			this.Controls.Add(this.btnScaleCities);
			this.Controls.Add(this.picMap);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "TravelingSalesmanProblem";
			this.Text = "Traveling Salesman Problem via SetBuilder Algorithm";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.TravelingSalesmanProblem_Closing);
			((System.ComponentModel.ISupportInitialize)(this.timerRefresh)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.nudSize)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.nudThreshold)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.timerStatusText)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		[STAThread]
		static void Main() {
			Application.Run(new TravelingSalesmanProblem());
		}

		#endregion

		#region Button-click event handlers

		private void btnInitialize_Click(object sender, System.EventArgs e) {
			network.Initialize();
			SetLevelList();
			picMap.Refresh();
		}

		private void btnClearNetwork_Click(object sender, System.EventArgs e) {
			network.Clear();
			network.Initialize();
			picMap.Refresh();
		}

		private void btnSolveStep_Click(object sender, System.EventArgs e) {
			network.Solve();
			SetLevelList();
			picMap.Refresh();
		}

		private void btnSolveFull_Click(object sender, System.EventArgs e) {
			if ((thread == null) && (network.cities.Count > 2)) {
				nudSize.Value = Math.Min(nudSize.Value, network.cities.Count * (network.cities.Count - 1));  // attempt to reduce this to a valid number to avoid bottom-level deadlocks.
				thread = new Thread(new ThreadStart(SolveFull));
				thread.Start();
				this.btnSolveFull.Text = "Abort";
				btnSolveFull.Refresh();
			}
			else {
				thread.Abort();
				thread = null;
				btnSolveFull.Text = "Solve (Full)";
				btnSolveFull.Refresh();
				// metrics
				decimal count = 0;
				decimal fitnessAverage = 0;
				foreach (ArrayList l in network.lists) {
					count += l.Count;
					foreach (Entity en in l)
						fitnessAverage += en.fitness;
				}
				fitnessAverage /=  count;
				txtStatus.Text = "Status: Stopped - items = " + count + " average fitness = " + fitnessAverage.ToString("###.#");
				txtStatus.Refresh();
			}
		}

		private void btnScaleCities_Click(object sender, System.EventArgs e) {
			// find largest coordinates
			double dMaxX = 0;
			double dMaxY = 0;
			double dMinX = picMap.Width;
			double dMinY = picMap.Height;
			foreach (object cityobject in network.cities) {
				dMaxX = Math.Max(dMaxX, ((City) cityobject).x);
				dMaxY = Math.Max(dMaxY, ((City) cityobject).y);
				dMinX = Math.Min(dMinX, ((City) cityobject).x);
				dMinY = Math.Min(dMinY, ((City) cityobject).y);
			}
			if (dMaxX == dMinX) {
				dMinX = 0;
				dMaxX = picMap.Width / 2;
			}
			else
				dMaxX = (picMap.Width - 40) / (dMaxX - dMinX);
			if (dMaxY == dMinY) {
				dMinY = 0;
				dMaxY = picMap.Height / 2;
			}
			else
				dMaxY = (picMap.Height - 40) / (dMaxY - dMinY);
			// adjust cities
			foreach (object cityobject in network.cities) {
				City city = (City) cityobject;
				city.x = 20 + (city.x - dMinX) * dMaxX;
				city.y = 20 + (city.y - dMinY) * dMaxY;
			}
			// show new coords, etc.
			network.CalculateDistances();
			network.Initialize();
			picMap.Refresh();
		}

		private void nudSize_ValueChanged(object sender, System.EventArgs e) {
			network.iMaxSize = (int) nudSize.Value;
			network.Initialize();
		}

		private void nudThreshold_ValueChanged(object sender, System.EventArgs e) {
			network.dThreshold = nudThreshold.Value;
		}

		#endregion

		#region Other event handlers

		private void picMap_Paint(object sender, System.Windows.Forms.PaintEventArgs e) {
			Font font = new Font("Times New Roman", 7.0f);
			StringFormat format = new StringFormat();
			format.Alignment = StringAlignment.Center;
			Graphics g = e.Graphics;
			g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
			g.FillRectangle(System.Drawing.Brushes.BlanchedAlmond, e.ClipRectangle);
			// draw best route
			if (network.lists != null) {
				for (int j = network.lists.Length - 1; j >= 0; j--) {
					if (network.lists[j].Count > 0) {
						Entity r = (Entity) network.lists[j][0];
						int iTotal = (int) (-r.fitness);
						string path = network.Assemble(r);
						int iHighlightStart = (int) (System.DateTime.Now.Ticks % iTotal);
						int iHighlightTotal = 40;  // total distance to highlight
						for (int i = 0; i < r.length; i++) {
							City a = ((City) network.cities[(int) path[i % r.length]]);
							City b = ((City) network.cities[(int) path[(i + 1) % r.length]]);
							int iCurrentLeg = (int) Math.Sqrt(Math.Pow(a.x - b.x, 2) + Math.Pow(a.y - b.y, 2));
							g.DrawLine(Pens.Black, (int) a.x, (int) a.y, (int) b.x, (int) b.y);
							if (iCurrentLeg < iHighlightStart)
								iHighlightStart -= iCurrentLeg;
							else {  // must highlight at least part of this leg
								if (iCurrentLeg >= iHighlightStart + iHighlightTotal) {
								}
								else {
								}
							}
						}
						break;
					}
				}
			}
			// draw cities
			foreach (object cityobject in network.cities) {
				City city = (City) cityobject;
				g.FillEllipse(Brushes.Tan, (int) (city.x - 5), (int) (city.y - 5), 10, 10);
				g.DrawEllipse(Pens.Black, (int) (city.x - 5), (int) (city.y - 5), 10, 10);
				g.DrawString(city.name, font, Brushes.Black, (int) (city.x), (int) (city.y - 20), format);
			}
		}

		private void picMap_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e) {
			iMouseX = e.X;
			iMouseY = e.Y;
			if (movingCity != null) {
				movingCity.x = iMouseX;
				movingCity.y = iMouseY;
				network.CalculateDistances();
				picMap.Refresh();
			}
		}

		private void picMap_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e) {
			City clickedCity = null;
			// check location first
			foreach (object cityobject in network.cities) {
				City city = (City) cityobject;
				if (Math.Abs(city.x - iMouseX) + Math.Abs(city.y - iMouseY) < 15) {  // close enough - delete city.
					clickedCity = city;
					break;
				}
			}
			if (e.Button == MouseButtons.Left) {  // deleting or placing city
				if (clickedCity != null)
					network.RemoveCity(clickedCity);
				else {  // placing new city
					int x = iMouseX;
					int y = iMouseY;  // preserve coordinates
					network.AddCity(x, y);
				}
				network.CalculateDistances();
				network.Initialize();
			}
			else if ((e.Button == MouseButtons.Right) && (clickedCity != null))  // start moving city
				movingCity = clickedCity;
			picMap.Refresh();
		}

		private void picMap_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e) {
			if ((e.Button == MouseButtons.Right) && (movingCity != null)) {
				movingCity = null;
				network.CalculateDistances();
				picMap.Refresh();
			}
		}

		private void picMap_MouseLeave(object sender, System.EventArgs e) {
			movingCity = null;
		}

		private void TravelingSalesmanProblem_Closing(object sender, System.ComponentModel.CancelEventArgs e) {
			if (thread != null)
				thread.Abort();
		}

		private void timerRefresh_Elapsed(object sender, System.Timers.ElapsedEventArgs e) {
			if (thread != null) {
				SetLevelList();
				picMap.Refresh();
			}
		}

		private void timerStatusText_Elapsed(object sender, System.Timers.ElapsedEventArgs e) {
			if (thread != null)
				txtStatus.Refresh();
		}
		#endregion

		#region Helper functions

		private void SetLevelList() {
			lstLevels.BeginUpdate();
			lstLevels.Items.Clear();
			for (int i = 0; i < network.lists.Length; i++) {
				int best = 0;
				double average = 0;
				if (network.lists[i].Count > 0) {
					best = (int) network.CalculateEntityDistance((Entity) network.lists[i][0]);
					for (int j = 0; j < network.lists[i].Count; j++)
						average += (int) network.CalculateEntityDistance((Entity) network.lists[i][j]);
					average /= network.lists[i].Count;
				}
				lstLevels.Items.Insert(0, new ListViewItem(new string[] { "" + (i + 2), "" + network.lists[i].Count, "" + best, "" + average.ToString("###.#") } ) );
			}
			lstLevels.EndUpdate();
			lstLevels.Refresh();
		}

		private void SolveFull() {
			int steps = 0;
			while(true) {
				steps++;
				network.Solve();
				// metrics
				decimal count = 0;
				decimal fitnessAverage = 0;
				foreach (ArrayList l in network.lists) {
					count += l.Count;
					foreach (Entity e in l)
						fitnessAverage += e.fitness;
				}
				fitnessAverage /= count;
				txtStatus.Text = "Status: Running - steps = " + steps + " items = " + count + " average fitness = " + fitnessAverage.ToString("###.#");
			}
		}

		#endregion

	}
}
